/*============================================================================
	File:		0010 - create demo databases.sql

	Summary:	This script creates creates a new database and restores the
				demo database STATISTICSDB for the demonstration of the 
				ASCENDING KEY PROBLEM!

				ATTENTION: The restored database STATISTICSDB must be in the
				compatibility level 110 for demonstration. Otherwise it will
				not work!

				THIS SCRIPT IS PART OF THE TRACK: "SQL Server Statistics"

	Date:		November 2016

	SQL Server Version: 2012 / 2014 / 2016
------------------------------------------------------------------------------
	Written by Uwe Ricken, db Berater GmbH

	This script is intended only as a supplement to demos and lectures
	given by Uwe Ricken.  
  
	THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
	ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
	TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
	PARTICULAR PURPOSE.
============================================================================*/
USE master;
GO

USE master;
GO

IF DB_ID(N'CustomerOrders') IS NOT NULL
BEGIN
	ALTER DATABASE CustomerOrders SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE CustomerOrders;
END
GO

DECLARE	@DataPath	NVARCHAR(256) = CAST(SERVERPROPERTY('InstanceDefaultDataPath') AS NVARCHAR(256)) + N'CustomerOrders.mdf';
DECLARE @LogPath	NVARCHAR(256) = CAST(SERVERPROPERTY('InstanceDefaultLogPath') AS NVARCHAR(256)) + N'CustomerOrders.ldf';

RESTORE DATABASE CustomerOrders
FROM DISK = N'S:\Backup\CustomerOrders.bak'
WITH
	MOVE N'CustomerOrders_Data' TO @DataPath,
	MOVE N'CustomerOrders_Log' TO @LogPath,
	STATS = 10,
	REPLACE,
	RECOVERY;
GO

ALTER AUTHORIZATION ON DATABASE::CustomerOrders TO sa;
ALTER DATABASE CustomerOrders SET RECOVERY SIMPLE;
GO

DECLARE	@DataPath	NVARCHAR(256) = CAST(SERVERPROPERTY('InstanceDefaultDataPath') AS NVARCHAR(256)) + N'demodb.mdf';
DECLARE @LogPath	NVARCHAR(256) = CAST(SERVERPROPERTY('InstanceDefaultLogPath') AS NVARCHAR(256)) + N'demodb.ldf';

IF DB_ID(N'demo_db') IS NOT NULL
BEGIN
	RAISERROR (N'Dropping database [demo_db]...', 0, 1) WITH NOWAIT;
	ALTER DATABASE demo_db SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE demo_db;
END

RAISERROR (N'Creating database [demo_db]...', 0, 1) WITH NOWAIT;
CREATE DATABASE demo_db;
GO

RAISERROR (N'Configuring database [demo_db]...', 0, 1) WITH NOWAIT;
ALTER DATABASE SCOPED CONFIGURATION SET LEGACY_CARDINALITY_ESTIMATION = OFF;
ALTER DATABASE demo_db SET RECOVERY SIMPLE;
ALTER AUTHORIZATION ON DATABASE::demo_db TO sa;
GO

DECLARE	@DataPath	NVARCHAR(256) = CAST(SERVERPROPERTY('InstanceDefaultDataPath') AS NVARCHAR(256)) + N'StatisticsDB.mdf';
DECLARE @LogPath	NVARCHAR(256) = CAST(SERVERPROPERTY('InstanceDefaultLogPath') AS NVARCHAR(256)) + N'StatisticsDB.ldf';

RAISERROR ('Restoring database STATISTICSDB to %s...', 0, 1, @DataPath) WITH NOWAIT;
IF DB_ID('STATISTICSDB') IS NOT NULL
BEGIN
	ALTER DATABASE STATISTICSDB SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE STATISTICSDB;
END

RESTORE DATABASE STATISTICSDB FROM DISK = N'S:\Backup\STATISTICS_2014.bak'
WITH
	MOVE 'Statistics_Data' TO @DataPath,
	MOVE 'Statistics_Log' TO @LogPath,
	STATS = 10;
GO

ALTER AUTHORIZATION ON DATABASE::STATISTICSDB TO sa;
ALTER DATABASE STATISTICSDB SET COMPATIBILITY_LEVEL = 110;
ALTER DATABASE STATISTICSDB SET RECOVERY SIMPLE;
ALTER DATABASE STATISTICSDB SET AUTO_UPDATE_STATISTICS OFF;
GO

USE demo_db;
GO

IF OBJECT_ID(N'dbo.PrepareWorkbench', N'P') IS NOT NULL
	DROP PROC dbo.PrepareWorkBench
	GO

CREATE PROC dbo.PrepareWorkbench
	@create_table	BIT = 1,
	@fill_table		BIT = 1,
	@language_id	INT	= 0
AS
	SET NOCOUNT ON;


	IF OBJECT_ID(N'dbo.messages', N'U') IS NOT NULL
	BEGIN
		RAISERROR (N'dropping existing table...', 0, 1) WITH NOWAIT;
		DROP TABLE dbo.messages;
	END

	IF @create_table = 0
		RETURN;

	IF @create_table = 1
	BEGIN
		RAISERROR (N'creating demo table [dbo].[messages]...', 0, 1) WITH NOWAIT;

		CREATE TABLE dbo.messages
		(
			message_id		INT,
			language_id		INT,
			severity		TINYINT,
			is_event_logged	BIT,
			[text]			CHAR(2048)
		);
	END

	IF @fill_table = 1
	BEGIN
		RAISERROR (N'filling demo table with data...', 0, 1) WITH NOWAIT;

		INSERT INTO dbo.messages WITH (TABLOCK)
		(message_id, language_id, severity, is_event_logged, [text])
		SELECT	message_id,
				language_id,
				severity,
				is_event_logged,
				[text]
		FROM	sys.messages
		WHERE	language_id = @language_id
				OR (@language_id = 0);
	END

	SET NOCOUNT OFF;
GO
