USE master;
GO

IF DB_ID('Statistics') IS NOT NULL
BEGIN
	ALTER DATABASE [Statistics] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [Statistics];
END
GO

CREATE DATABASE [Statistics]
ON PRIMARY
(
	NAME = 'Statistics_Data',
	SIZE = 500MB,
	FILEGROWTH = 100MB,
	FILENAME = N'F:\DATA\Statistics.mdf'
)
LOG ON
(
	NAME = 'Statistics_Log',
	SIZE = 500MB,
	FILEGROWTH = 100MB,
	FILENAME = N'F:\LOG\Statistics.ldf'
);
GO

ALTER DATABASE [Statistics] SET COMPATIBILITY_LEVEL = 110;
ALTER DATABASE [Statistics] SET RECOVERY SIMPLE;
ALTER AUTHORIZATION ON DATABASE::[Statistics] TO sa;
GO

USE [Statistics];
GO

CREATE TABLE dbo.Orders
(
	ID			INT		NOT NULL	IDENTITY (1, 1),
	CUSTID		INT		NOT NULL,
	ORDERDATE	DATE	NOT NULL,
	AMOUNT		MONEY	NOT NULL,
	NOTE		CHAR(100)	NOT NULL	DEFAULT ('Only a filler')
);
GO

-- NO FILL THE DATA FROM
--	0040 - SQL Server Statistics - Add 10.000.000 orders

CREATE UNIQUE CLUSTERED INDEX cuix_Orders_Id ON dbo.Orders (Id);
CREATE NONCLUSTERED INDEX nix_Orders_CUSTID ON dbo.Orders (CustId);
CREATE NONCLUSTERED INDEX nix_Orders_OrderDate ON dbo.Orders (OrderDate);
GO

-- NOW FILL ADDITIONAL DATA FROM
--	0050 - SQL Server Statistics - Add 1.000.000 orders

BACKUP DATABASE [Statistics] TO DISK = N'S:\BACKUP\STATISTICS.BAK'
WITH STATS, INIT, FORMAT, COMPRESSION;
GO
